'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _getVars = require('./get-vars.js');

var _getVars2 = _interopRequireDefault(_getVars);

var _util = require('../../util.js');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }
/* eslint-disable no-use-before-define */


var renderIf = function renderIf(ast, depth) {
  var predicate = ast.predicate,
      body = ast.body,
      elseBody = ast.elseBody;


  var elseString = render(elseBody, depth + 1);
  var elseLines = elseString ? [_util2.default.indent('} else {', depth), elseString] : [];
  return ['if (' + render(predicate) + ') {', render(body, depth + 1)].concat(elseLines, [_util2.default.indent('}', depth)]).join('\n');
};

var renderFor = function renderFor(ast, depth) {
  var init = ast.init,
      condition = ast.condition,
      loop = ast.loop,
      body = ast.body;

  return ['for (' + render(init) + '; ' + render(condition) + '; ' + render(loop) + ') {', render(body, depth + 1), _util2.default.indent('}', depth)].join('\n');
};

var renderForIn = function renderForIn(ast, depth) {
  var variable = ast.variable,
      iterator = ast.iterator,
      body = ast.body;

  return ['for (var ' + render(variable) + ' in ' + render(iterator) + ') {', render(body, depth + 1), _util2.default.indent('}', depth)].join('\n');
};

var renderFunction = function renderFunction(ast, depth) {
  var name = ast.name,
      argument = ast.argument,
      body = ast.body;

  var vars = (0, _getVars2.default)(body);
  var varLines = vars.length === 0 ? [] : [_util2.default.indent('var ' + vars.join(', ') + ';', depth + 1)];
  return ['function ' + render(name) + '(' + render(argument) + ') {'].concat(varLines, [render(body, depth + 1), _util2.default.indent('}', depth)]).join('\n');
};

var renderUnop = function renderUnop(ast, depth) {
  var child = '(' + render(ast.child) + ')';
  if (ast.style === 'prefix') {
    return '' + ast.op + child;
  } else {
    return '' + child + ast.op;
  }
};

var renderObjectLiteral = function renderObjectLiteral(ast, depth) {
  var object = ast.object;

  var lines = _lodash2.default.map(object, function (value, key) {
    var valueString = render(value, depth + 1);
    return _util2.default.indent(key + ': ' + valueString.trimLeft() + ',', depth + 1);
  });
  return [_util2.default.indent('{', depth)].concat(_toConsumableArray(lines), [_util2.default.indent('}', depth)]).join('\n');
};

var STATEMENTS_WITH_SEMIS = ['assignment', 'return', 'binop', 'call', 'unop'];

var render = function render(ast) {
  var depth = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

  switch (ast.type) {
    case 'assignment':
      return render(ast.variable) + ' = ' + render(ast.value);
    case 'if':
      return renderIf(ast, depth);
    case 'return':
      return 'return ' + render(ast.value, depth).trimLeft();
    case 'body':
      return _lodash2.default.map(ast.body, function (s) {
        var suffix = _lodash2.default.includes(STATEMENTS_WITH_SEMIS, s.type) ? ';' : '';
        var line = _util2.default.indent(render(s, depth), depth);
        return '' + line + suffix;
      }).join('\n');
    case 'for':
      return renderFor(ast, depth);
    case 'forin':
      return renderForIn(ast, depth);
    case 'empty':
      return '';
    case 'function1':
      return renderFunction(ast, depth);
    case 'binop':
      return render(ast.left, depth) + ' ' + ast.comparator + ' ' + render(ast.right, depth);
    case 'var':
      return ast.value;
    case 'literal':
      return ast.value;
    case 'call':
      return render(ast.fn) + '(' + render(ast.arg) + ')';
    case 'unop':
      return renderUnop(ast, depth);
    case 'objectliteral':
      return renderObjectLiteral(ast, depth);
    case 'propertyaccess':
      return render(ast.obj, depth) + '.' + ast.property;
    case 'bracketaccess':
      return render(ast.obj, depth) + '[' + render(ast.property, depth) + ']';
    case 'typeof':
      return 'typeof ' + render(ast.child, depth);
    case 'comment':
      return '/* ' + ast.comment + ' */';
    case 'error':
      return '';
    default:
      throw new Error('Unexpected AST: ' + ast);
  }
};

exports.default = render;